<?php
/*
file: chartFunctions.php

Copyright (c) 2009, Shaun Henry. All rights reserved. 
 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions 
 are met: 
 1. Redistributions of source code must retain the above copyright 
 notice, this list of conditions and the following disclaimer. 
 2. Redistributions in binary form must reproduce the above copyright 
 notice, this list of conditions and the following disclaimer in the 
 documentation and/or other materials provided with the distribution. 
 3. The name of the author may not be used to endorse or promote
 products derived from this software without specific prior
 written permission.  
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
*/



include './ofc/php-ofc-library/open-flash-chart.php';

$minimumBarChartWidth = 400;
$maximumBarChartWidth = 1200;

$minimumBarChartHeight = 400;
$maximumBarChartHeight = 800;

/******************************
OFC chart functions
*******************************/

/**
get the maximum numeric value int he array and return it
param -  valueArray: an array with numeric values in it
return - the largest value in the array
*/
function getMaxValueFromArray($valueArray)
{
  $maxvalue = $valueArray[0];
  foreach($valueArray as $value)
  {
    if($value > $maxValue)
	{
	  $maxValue = $value;
	}
  }
  return $maxValue;
}

/**
get the width of the bar chart depending on the number of data points
param - numDataPoints: the number of data points
return - the width of the bar chart
*/
function getBarChartWidth($numDataPoints)
{
  global $minimumBarChartWidth;
  global $maximumBarChartWidth;  
  $width = $numDataPoints * 20;
  if($width < $minimumBarChartWidth)
  {
    $width = $minimumBarChartWidth;
  }
  if($width > $maximumBarChartWidth)
  {
    $width = $maximumBarChartWidth;
  }
  return $width;
}

/**
get the height of the bar chart depending on the number of data points
param - numDataPoints: the number of data points
return - the height of the bar chart
*/
function getBarChartHeight($numDataPoints)
{
  global $minimumBarChartHeight;
  global $maximumBarChartHeight;
  
  $height = $numDataPoints * 20;
  if($height < $minimumBarChartHeight)
  {
    $height = $minimumBarChartHeight;
  }
  if($height > $maximumBarChartHeight)
  {
    $height = $maximumBarChartHeight;
  }
  return $height;
}

/**
Generate the HTML code to display a simple bar chart (where the series is laid out from left to right). 
param -  referenceName: A unique name for this chart.  This is used by the OFC flash application to load the chart data.
param -  title: The title of the chart.  Will be displayed at the top of the chart
param -  dataArray: An array containing the data to display
param -  dataLabelArray: An array containing the labels for each of the series elements (should be the same size as the dataArray)
param -  tooltipArray: An array containing the mouse-hover tooltips for each of the series elements (should be the same size as the dataArray)
param -  xChartSize: The width of the chart in pixels
param -  yChartSize: The height of the chart in pixels
param -  verticalXAxisLabels: forces the labels to be displayed vertically at the bottom of the chart.  Set to true when there are lots of datapoints.
return -  Returns HTML code suitable to be output directly in an HTML page.
*/
function getSimpleBarChart($referenceName, $title, $dataArray, $dataLabelArray, $tooltipArray, $xChartSize, $yChartSize, $verticalXAxisLabels)
{
  //generate the chart
  $referenceName = "chart_".$referenceName;
  $title = new title($title);
  $bar = new bar();
  
  $valuesArray = array();
  $index = 0;
  foreach($dataArray as $data)
  {
    $value = new bar_value($data);
    if(isset($tooltipArray) && $tooltipArray != "")
    {
      $value->set_tooltip($tooltipArray[$index]);
      $value->on_click = "alert('you clicked me')";
    }
    array_push($valuesArray, $value);
    $index++;
  }
  
  $bar->set_values($valuesArray);
  $xLabels = new x_axis_labels();
  $xLabels->set_labels($dataLabelArray);
  if($verticalXAxisLabels == true)
  {
    $xLabels->set_vertical();
  }
  $xAxis = new x_axis();
  $xAxis->set_labels($xLabels);
  $yAxis = new y_axis();
  $maxValue = getMaxValueFromArray($dataArray);
  $yAxisLabelInterval = floor($maxValue / floor($yChartSize / 20));
  if($yAxisLabelInterval < 5)
  {
    $yAxisLabelInterval = 5;
  }
  $yAxis->set_range(0, $maxValue + $yAxisLabelInterval, $yAxisLabelInterval);
  $chart = new open_flash_chart();
  $chart->set_title( $title );
  $chart->add_element( $bar );
  $chart->set_x_axis($xAxis);
  $chart->set_y_axis($yAxis);  
  
  return getChartOutput($referenceName, $chart, $xChartSize, $yChartSize);
}

/**
Generate the HTML code to display a simple pie chart. 
param -  referenceName: A unique name for this chart.  This is used by the OFC flash application to load the chart data.
param -  title: The title of the chart.  Will be displayed at the top of the chart
param -  dataArray: An array containing the data to display
param -  dataLabelArray: An array containing the labels for each of the series elements (should be the same size as the dataArray)
param -  xChartSize: The width of the chart in pixels
param -  yChartSize: The height of the chart in pixels
return -  Returns HTML code suitable to be output directly in an HTML page.
*/
function getSimplePieChart($referenceName, $title, $dataArray, $dataLabelArray, $xChartSize, $yChartSize)
{
  //generate the chart
  $referenceName = "chart_".$referenceName;
  $title = new title($title);
  $pie = new pie();
  $pie->set_start_angle( 35 );
  $pie->set_animate( true );
  $pie->set_tooltip( '#val# of #total#<br>#percent# of 100%' );
  $valueLabelArray = array();
  $index = 0;
  foreach($dataArray as $data)
  {
    array_push($valueLabelArray, new pie_value($data, $dataLabelArray[$index]));
	$index++;
  }
  $pie->set_values($valueLabelArray);
  $chart = new open_flash_chart();
  $chart->set_title($title);
  $chart->add_element($pie);
  $chart->x_axis = null;
  
  return getChartOutput($referenceName, $chart, $xChartSize, $yChartSize);
}

/**
Generate the HTML/Javascript code to display the given chart
param -  referenceName: A unique name for this chart.  This is used by the OFC flash application to load the chart data.
param -  chart: an OFC chart object that has already been configured
param -  xChartSize: The width of the chart in pixels
param -  yChartSize: The height of the chart in pixels
return -  HTML/javascript code suitable for inclusion in an HTML file.
*/
function getChartOutput($referenceName, $chart, $xChartSize, $yChartSize)
{
  $output  = "<script type=\"text/javascript\">\r\n";
  $output .= "  function get_data_".$referenceName."()\r\n";
  $output .= "  {\r\n";
  $output .= "    //alert( 'reading data' );\r\n";
  $output .= "    return JSON.stringify(data_".$referenceName.");\r\n";
  $output .= "  }\r\n";
  $output .= "  swfobject.embedSWF(\"./ofc/open-flash-chart.swf\", \"".$referenceName."\", \"".$xChartSize."\", \"".$yChartSize."\", \"9.0.0\", \"expressInstall.swf\", {\"get-data\":\"get_data_".$referenceName."\",\"loading\":\"Loading chart...\"} );\r\n";
  $output .= "  var data_".$referenceName." = ".$chart->toPrettyString().";\r\n";
  $output .= "</script>\r\n";
  $output .= "<div id=\"".$referenceName."\"></div>\r\n";
  return $output;
}

?>